@@ -21,12 +21,33 @@
 # along with ConfigViper. If not, see <http://www.gnu.org/licenses/>.
 #
 
+import codecs
+import hashlib
 import json
+import logging
 import os
 import pprint
 import shutil
+import socket
+import threading
+import warnings
 
 import converters
+from lockfile import lockfile
+
+
+try:
+    from logging import NullHandler
+except ImportError:
+    # keeps Python 2.6 working
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
+
+ROOT_LOGGER = 'configviper'
+
+logging.getLogger(ROOT_LOGGER).addHandler(NullHandler())
+
 
 class PathOverrun(Exception):
     """This exception will be raised by :meth:`~ConfigViper.set` method when a
@@ -139,16 +160,17 @@ class ConfigViper(object):
     _sv_converters = {}
     """Converters dict."""
 
-    _sv_make_backups = True
-    """..."""
-
-    _sv_keep_backups = False
-    """..."""
+    _sv_encoding = 'utf-8'
+    """Encoding which is to be used for the file."""
 
+    _sv_ignore_encoding = False
+    """If ignore encoding the JSON file will be opened with the Python's
+    built-in ``open`` function. Otherwise the file will be opened with
+    the ``codecs.open``."""
 
     @staticmethod
-    def configure(pathname='', filename='', separator='.',
-            auto_save=True, make_backups=True, keep_backups=False):
+    def configure(pathname='', filename='', separator='.', 
+            auto_save=True, encoding='utf-8', ignore_encoding=False, **kwargs):
         """Set parameters that will be used by all instances of ``ConfigViper``.
         This method is intended to be called only once, when the host 
         application is preparing everything up to be ran.
@@ -167,22 +189,32 @@ class ConfigViper(object):
             configuration file will be automatically saved. 
             Defaults to ``True``.
 
-        :param make_backups: Make a backup of configuration file right before
-            save operation. The backup filename should be created on the same
-            directory of original configuration file with the ``.backup``
-            appended. Defaults to ``True``.
+        :param encoding: String. Encoding which is to be used for the file.
+            Defaults to ``"UTF-8"``.
 
-        :param keep_backups: Keep backup file after successful save.
-            Defaults to ``False``.
+        :param ignore_encoding: Boolean. If ignore encoding the JSON file will
+            be opened with the Python's built-in ``open`` function. Otherwise,
+            the file will be opened with the ``codecs.open``, wich is the
+            default behavior.
 
         """
+        if 'make_backups' in kwargs:
+            warnings.warn('ConfigViper: parameter make_backups ignored. '
+                    'In release 0.6 the presence of this parameter will '
+                    'raise an error.')
+
+        if 'keep_backups' in kwargs:
+            warnings.warn('ConfigViper: parameter keep_backups ignored. '
+                    'In release 0.6 the presence of this parameter will '
+                    'raise an error.')
+
         ConfigViper._sv_pathname = pathname
         ConfigViper._sv_filename = filename
         ConfigViper._sv_path_separator = separator
         ConfigViper._sv_save_on_set = bool(auto_save)
-        ConfigViper._sv_make_backups = make_backups
-        ConfigViper._sv_keep_backups = keep_backups
         ConfigViper._sv_converters = {}
+        ConfigViper._sv_encoding = encoding
+        ConfigViper._sv_ignore_encoding = ignore_encoding
         
         if not ConfigViper._sv_pathname:
             # set default path name
@@ -202,23 +234,34 @@ class ConfigViper(object):
         self._pathname = ConfigViper._sv_pathname
         self._filename = ConfigViper._sv_filename
         self._confname = os.path.join(self._pathname, self._filename)
-        self._backupname = os.path.join(
-                self._pathname, '%s.backup' % self._filename)
-        
         self._save_on_set = bool(ConfigViper._sv_save_on_set)
         self._stop_saving__save_on_set_flag = self._save_on_set
         self._path_separator = ConfigViper._sv_path_separator
-        self._make_backups = ConfigViper._sv_make_backups
-        self._keep_backups = ConfigViper._sv_keep_backups
-        
+        self._encoding = ConfigViper._sv_encoding
+        self._ignore_encoding = ConfigViper._sv_ignore_encoding
+
         # ensure path name exists
         if not os.path.exists(self._pathname):
             os.makedirs(self._pathname)
             
         # if config file exists then load
         if os.path.exists(self._confname):
-            with open(self._confname, 'rb') as f:
-                self._config_map = json.load(f)
+            try:
+                logger = logging.getLogger(ROOT_LOGGER)
+                if self._ignore_encoding:
+                    with open(self._confname, 'rb') as f:
+                        self._config_map = json.load(f)
+                else:
+                    with codecs.open(self._confname, 'rb', 
+                            encoding=self._encoding) as f:
+                        self._config_map = json.load(f, self._encoding)
+            except:
+                logger.exception('error loading existing file '
+                        '("%s", encoding="%s", ignore_encoding=%s)',
+                                self._confname, 
+                                self._encoding, 
+                                self._ignore_encoding)
+                raise
 
 
     def __repr__(self):
@@ -319,47 +362,103 @@ class ConfigViper(object):
         """
         self._save_on_set = True
         self._stop_saving__save_on_set_flag = True
-            
-        
+    
+
     def save(self):
-        """Save configurations in JSON format, with indentation and 
-        sorted keys. If :attr:`is_make_backups` is ``True`` a backup file will
-        be created right before the actual save. In case of failure, the
-        backup will be restored and is guaranteed that no exceptions will be
-        silenced on any failure.
+        """Save configurations formatted as JSON to a file. This method attempts
+        to be as safe as possible, using a strategy based on a temporary 
+        directory that is created in the same directory as the configuration
+        file itself. During the save process, the directory will looks like:
+
+        .. code-block:: text
+
+            ~/.configviper/configviper.json
+            ~/.configviper/configviper-4b1b7b5f4a594926ac82de8ecaa439c4/c8c.backup
+            ~/.configviper/configviper-4b1b7b5f4a594926ac82de8ecaa439c4/c8c.temp
+
+        Where the ``configviper.json`` is the actual configuration file. 
+        The ``configviper-4b1b7b5f4a594926ac82de8ecaa439c4`` is a temporary
+        directory name that is unique to the current process.
+        The file ``c8c.backup`` is a copy of ``configviper.json`` and the file
+        ``c8c.temp`` is the file where the configuration data was actually
+        saved. With all that in place, the ``.temp`` file will be copied back to
+        the original configuration file. If everything goes ok, the temporary
+        directory will be wiped out and the save is done. If not, the backup
+        file will be restored and the temporary directory will be wiped out.
         """
-        # restore the save on set flag; this ensures that the default
-        # behavior is restored even if any error occurs;
+        # restore the save on set flag; this restore the expected behavior
+        # if stop_saving() has been called before;
         self._save_on_set = self._stop_saving__save_on_set_flag
 
-        # make a backup before save (if required)...
-        if self._make_backups:
-            if os.path.exists(self._confname):
-                shutil.copyfile(self._confname, self._backupname)
+        logger = logging.getLogger(ROOT_LOGGER)
 
-        # convert configurations dict to json;
-        data = json.dumps(self._config_map, sort_keys=True, indent=4)
+        temp_dir = self.get_unique_dir_name()
+        temp_filename = self.get_unique_temp_filename()
+        temp_backup = self.get_unique_backup_filename()
 
         try:
-            with open(self._confname, 'wb') as f:
-                f.write(data)
+            # remove temp dir (if already exists)
+            if os.path.isdir(temp_dir):
+                logger.debug('preparing for save: remove existing temp dir...')
+                shutil.rmtree(temp_dir)
 
-        except:
-            # ouch! try to restore backup file (if there is one) and 
-            # honor keep_backups option
-            if self._make_backups and os.path.exists(self._backupname):
-                shutil.copyfile(self._backupname, self._confname)
-                if not self._keep_backups:
-                    os.unlink(self._backupname)
+            # create temp dir (w unique name for the current process)
+            logger.debug('preparing for save: creating temp dir...')
+            os.mkdir(temp_dir)
 
-            # and re-raise exception
-            raise
+            # make a backup for the current file (if we have one)
+            if os.path.isfile(self._confname):
+                logger.debug('preparing for save: making backup...')
+                shutil.copyfile(self._confname, temp_backup)
+
+            # write current data to a temp file
+            logger.debug('preparing for save: writing temp file...')
+            with lockfile(temp_filename, 'wb',
+                    encoding=self._encoding,
+                    ignore_encoding=self._ignore_encoding) as locker:
+                data = json.dumps(self._config_map, sort_keys=True, indent=4)
+                locker.file_object.write(data)
+
+            # copy the new (temp) file over the current one
+            logger.debug('saving: copying temp file to current file...')
+            shutil.copyfile(temp_filename, self._confname)
+
+        except BaseException as original_exception:
+            logger.exception('error saving '
+                    '(temp dir="%s", temp file="%s", backup file="%s")',
+                            temp_dir,
+                            temp_filename,
+                            temp_backup)
+            try:
+                # restore backup over the current configurations file and
+                # remove temp dir if succeeded
+                logger.debug('restoring backup "%s" over "%s"',
+                        temp_backup, self._confname)
+                shutil.copyfile(temp_backup, self._confname)
+
+                logger.debug('removing temp dir "%s"', temp_dir)
+                shutil.rmtree(temp_dir)
+            except:
+                # log the current exception...
+                logger.exception('error restoring backup after save failure '
+                        '(copying from "%s" to "%s")',
+                                temp_backup, 
+                                self._confname)
+
+            # ...and re-raise the original
+            raise original_exception
 
         else:
-            # finally, keep or delete the backup file (if any)
-            if self._make_backups and (not self._keep_backups):
-                if os.path.exists(self._backupname):
-                    os.unlink(self._backupname)
+            # at this point, save was successful
+            logger.debug('saving done!')
+
+            try:
+                if os.path.isdir(temp_dir):
+                    shutil.rmtree(temp_dir)
+            except:
+                # mutes any errors explicitly (just logs it)
+                logger.exception('muted error: removing temp dir after '
+                        'successful save (temp dir "%s")', temp_dir)
 
 
     def stop_saving(self):
@@ -469,6 +568,34 @@ class ConfigViper(object):
         return converter
 
 
+    def get_unique_dir_name(self):
+        # grab current thread ident
+        t = threading.current_thread()
+        ident = getattr(t, 'ident', hash(t))
+        identhex = '%x' % (ident & 0xffffffff)
+        
+        # collect other values
+        hostname = socket.gethostname()
+        pidvalue = str(os.getpid())
+        
+        # compound a string sequence based on collected values
+        sequence = 'configviper|%s|%s|%s' % (identhex, hostname, pidvalue)
+        unique_name = 'configviper-%s' % hashlib.md5(sequence).hexdigest()
+
+        path = os.path.dirname(os.path.abspath(self._confname))
+        return os.path.join(path, unique_name)
+        
+        
+    def get_unique_temp_filename(self):
+        filename = '%x.temp' % os.getpid()
+        return os.path.join(self.get_unique_dir_name(), filename)
+
+
+    def get_unique_backup_filename(self):
+        filename = '%x.backup' % os.getpid()
+        return os.path.join(self.get_unique_dir_name(), filename)
+
+
     @property
     def pathname(self):
         """The path name where configuration file should be saved by 
@@ -500,7 +627,10 @@ class ConfigViper(object):
         """The full path and file name for the backup. 
         See :attr:`is_make_backups` and :meth:`save`.
         """
-        return self._backupname
+        # TODO: remove this property in 0.6 release
+        warnings.warn('ConfigViper: backup_filename property will be'
+                'removed in 0.6 release.')
+        return '' # self._backupname
     
 
     @property
@@ -526,7 +656,10 @@ class ConfigViper(object):
         See :attr:`is_keep_backups`. The backup file will have the same name as
         the original configuration file with ``.backup`` appended.
         """
-        return self._make_backups
+        # TODO: remove this property in 0.6 release
+        warnings.warn('ConfigViper: is_make_backups property will be '
+                'removed in 0.6 release.')
+        return False # self._make_backups
 
 
     @property
@@ -534,5 +667,8 @@ class ConfigViper(object):
         """Indicate whether the backup file should be kept when the :meth:`save`
         method completes, even if an error occurs. See :attr:`is_make_backups`.
         """
-        return self._keep_backups
+        # TODO: remove this property in 0.6 release
+        warnings.warn('ConfigViper: is_keep_backups property will be '
+                'removed in 0.6 release.')
+        return False # self._keep_backups
 
