@@ -391,69 +391,6 @@ class TestStopSaving(unittest.TestCase):
             os.unlink(self._cfg_filename)
 
 
-class TestBackupFeature(unittest.TestCase):
-    """Test backup feature."""
-
-    def setUp(self):
-        self._cfg_filename = 'test-backup.json'
-        self._bkp_filename = '%s.backup' % self._cfg_filename
-
-        ConfigViper.configure(
-                pathname='.', 
-                filename=self._cfg_filename,
-                keep_backups=True)
-
-        # ensure file and their backup do not exists
-        if os.path.exists(self._cfg_filename):
-            os.unlink(self._cfg_filename)
-
-        if os.path.exists(self._bkp_filename):
-            os.unlink(self._bkp_filename)
-
-
-    def test_keep_backup(self):
-        """Test for the existence of backup file (keep backup option)."""
-        conf = ConfigViper()
-
-        self.assertTrue(conf.is_make_backups)
-        self.assertTrue(conf.is_keep_backups)
-
-        # assert the name of backup file
-        self.assertEqual(
-                os.path.join(conf.pathname, self._bkp_filename), 
-                conf.backup_filename)
-
-        # on first set save() will be called, but there's no configuration
-        # file to be backed up; on a second call to set, we can check for
-        # the existence of a backup file...
-        conf.set('spam.ham', 1)
-        conf.set('spam.eggs', 2)
-
-        self.assertTrue(os.path.exists(conf.backup_filename))
-
-        # the original file should contains both values...
-        with open(conf.config_filename, 'r') as f:
-            data = json.load(f)
-
-        self.assertEqual(1, data['spam']['ham'])
-        self.assertEqual(2, data['spam']['eggs'])
-
-        # the backup file only the first value
-        with open(conf.backup_filename, 'r') as f:
-            data = json.load(f)
-
-        self.assertEqual(1, data['spam']['ham'])
-        self.assertRaises(KeyError, lambda: data['spam']['eggs'])
-
-
-    def tearDown(self):
-        if os.path.exists(self._cfg_filename):
-            os.unlink(self._cfg_filename)
-        
-        if os.path.exists(self._bkp_filename):
-            os.unlink(self._bkp_filename)
-
-
 class SpamEggs(object):
 
     def __init__(self):
